# posthtml-ast-contains-only-empty-space

<a href="https://standardjs.com" style="float: right; padding: 0 0 20px 20px;"><img src="https://cdn.rawgit.com/feross/standard/master/sticker.svg" alt="Standard JavaScript" width="100" align="right"></a>

> Returns Boolean true/false depending if passed AST (nested arrays/strings/plain objects) contain only empty space (what would _.trim to nothing)

[![Link to npm page][npm-img]][npm-url]
[![Build Status][travis-img]][travis-url]
[![bitHound Overall Score][overall-img]][overall-url]
[![bitHound Dependencies][deps-img]][deps-url]
[![bitHound Dev Dependencies][dev-img]][dev-url]
[![Coverage Status][cov-img]][cov-url]
[![Known Vulnerabilities][vulnerabilities-img]][vulnerabilities-url]
[![Downloads/Month][downloads-img]][downloads-url]
[![View dependencies as 2D chart][deps2d-img]][deps2d-url]
[![Test in browser][runkit-img]][runkit-url]

> This is not a PostHTML plugin but a regular JS library. It is meant to work with anything, including nested array/object trees, like parsed HTML, that is, AST (abstract syntax trees) that come out of the PostHTML-parser. Hence the name, `posthtml-ast-`.

## Table of Contents

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Install](#install)
- [Use](#use)
- [Rationale](#rationale)
- [API](#api)
- [Testing](#testing)
- [Contributing](#contributing)
- [Licence](#licence)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Install

```bash
$ npm install --save posthtml-ast-contains-only-empty-space
```

## Use

```js
var empty = require('posthtml-ast-contains-only-empty-space')
...
// All values are empty, this means this object contains only empty space.
// Notice it's nested in an array, it does not matter.
console.log(empty([{ 'content': {} }]))
// => true

console.log(empty([{ 'tag': 'style' }]))
// => false

// Works on simple arrays as well:
console.log(empty(['   ', ' ']))
// => true

// Works on strings as well:
console.log(empty('   '))
// => true

// Object keys that have values as null are considered empty:
console.log(empty({a: null}))
// => true

// Works no matter how deeply nested input is:
console.log(empty(
  {
    a: [{
      x: {
        y: [
          {
            z: [
              '\n'
            ]
          }
        ]
      }
    }],
    b: ['\t\t\t  '],
    c: ['\n \n\n'],
    d: ['\t   ']
  }
))
// => true
```

## Rationale

Working with parsed HTML is always a battle against white space. Often you need to know, does certain AST piece (object/array/whatever) contain anything real, or just an empty space. That's what this library is for.

In real life, parsed HTML trees will have many levels of nested arrays, objects and strings. While it's easy to check does a plain object contain only empty space (`\n`, ` `, `\t`, line break or a mix of thereof), it's not so easy when your object has arrays of empty objects. I want a solid, tested library which can identify emptiness (or lack of) in anything, nested or not nested.

By the way, weird things (like functions, which don't belong to parsed HTML structures) will yield cause result `false`.

## API

```js
empty (input); // array, object or string — normally AST (which is array of nested objects/strings/arrays)
// => true/false
```

Returns Boolean `true` or `false` and does not mutate the input arguments.

## Testing

```bash
$ npm test
```

Unit tests use [AVA](https://github.com/avajs/ava) and [JS Standard](https://standardjs.com) notation.

## Contributing

All contributions are welcome. Please stick to [Standard JavaScript](https://standardjs.com) notation and supplement the `test.js` with new unit tests covering your feature(s).

If you see anything incorrect whatsoever, do [raise an issue](https://github.com/codsen/posthtml-ast-contains-only-empty-space/issues). If you file a pull request, I'll do my best to help you to get it merged in a timely manner. If you have any comments on the code, including ideas how to improve things, don't hesitate to contact me by email.

## Licence

> MIT License (MIT)

> Copyright (c) 2017 Codsen Ltd, Roy Revelt

> Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

> The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

> THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

[npm-img]: https://img.shields.io/npm/v/posthtml-ast-contains-only-empty-space.svg
[npm-url]: https://www.npmjs.com/package/posthtml-ast-contains-only-empty-space

[travis-img]: https://travis-ci.org/codsen/posthtml-ast-contains-only-empty-space.svg?branch=master
[travis-url]: https://travis-ci.org/codsen/posthtml-ast-contains-only-empty-space

[cov-img]: https://coveralls.io/repos/github/codsen/posthtml-ast-contains-only-empty-space/badge.svg?branch=master
[cov-url]: https://coveralls.io/github/codsen/posthtml-ast-contains-only-empty-space?branch=master

[overall-img]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space/badges/score.svg
[overall-url]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space

[deps-img]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space/badges/dependencies.svg
[deps-url]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space/master/dependencies/npm

[dev-img]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space/badges/devDependencies.svg
[dev-url]: https://www.bithound.io/github/codsen/posthtml-ast-contains-only-empty-space/master/dependencies/npm

[downloads-img]: https://img.shields.io/npm/dm/posthtml-ast-contains-only-empty-space.svg
[downloads-url]: https://www.npmjs.com/package/posthtml-ast-contains-only-empty-space

[vulnerabilities-img]: https://snyk.io/test/github/codsen/posthtml-ast-contains-only-empty-space/badge.svg
[vulnerabilities-url]: https://snyk.io/test/github/codsen/posthtml-ast-contains-only-empty-space

[deps2d-img]: https://img.shields.io/badge/deps%20in%202D-see_here-08f0fd.svg
[deps2d-url]: http://npm.anvaka.com/#/view/2d/posthtml-ast-contains-only-empty-space

[runkit-img]: https://img.shields.io/badge/runkit-test_in_browser-a853ff.svg
[runkit-url]: https://npm.runkit.com/posthtml-ast-contains-only-empty-space
